<?php

/**
 * @file
 * Contains Drupal\cacheflush\Controller\CacheflushApi.
 */

namespace Drupal\cacheflush\Controller;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Exception\HttpException;

/**
 * Returns responses for System Info routes.
 */
class CacheflushApi implements ContainerInjectionInterface {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static();
  }

  /**
   * Clear all caches.
   *
   * @see drupal_flush_all_caches().
   *
   * @return RedirectResponse
   */
  public function clearAll() {
    drupal_flush_all_caches();
    drupal_set_message(t('Cache cleared.'));
    return $this->redirectUrl();
  }

  /**
   * Clear cache preset by cacheflush entity id.
   *
   * @param int $cid
   *   Caheflush entity id to run.
   *
   * @return RedirectResponse
   */
  public function clearById($cid) {
    return $this->clearPresetCache($cid);
  }

  /**
   * Based on settings decide witch clear cache function to be called.
   *
   * @param int $cid
   *   Preset id to do clear cache for.
   */
  private function clearPresetCache($cid) {
    $entity = cacheflush_load($cid);

    $this->checkError($entity);

    \Drupal::moduleHandler()->invokeAll('cacheflush_before_clear', [$entity]);

    $presets = $entity->getData();
    if ($presets) {
      foreach ($presets as $cache) {
        foreach ($cache['functions'] as $value) {
          if (is_callable($value['#name'])) {
            call_user_func_array($value['#name'], $value['#params']);
          }
          else {
            \Drupal::logger('CACHEFLUSH')->warning(t("Function cannot be called: @name", array('@name' => $value['#name'])));
          }
        }
      }
    }

    drupal_set_message(t("All predefined cache options in @name was cleared.", array('@name' => $entity->getTitle())));
    \Drupal::moduleHandler()->invokeAll('cacheflush_after_clear', [$entity]);

    return $this->redirectUrl();
  }

  /**
   * Return a list of cache options to be cleared.
   *
   * @return array
   *   List cache options.
   */
  public function getOptionList() {
    $bins = $this->createTabOptions();
    $other = \Drupal::moduleHandler()->invokeAll('cacheflush_tabs_options');
    return array_merge($bins, $other);
  }

  /**
   * Create option array for preset.
   *
   * @return array
   */
  public function createTabOptions() {
    $container = \Drupal::getContainer();
    $core = $this->coreBinMapping();
    foreach ($container->getParameter('cache_bins') as $service_id => $bin) {
      $options[$bin] = array(
        'description' => t('Storage for the cache API.'),
        'category' => isset($core[$bin]) ? 'vertical_tabs_core' : 'vertical_tabs_custom',
        'functions' => array(
          '0' => array(
            '#name' => 'CacheflushApi::clearBinCache',
            '#params' => array($service_id),
          ),
        ),
      );
    }
    return $options;
  }

  /**
   * Clear cache by service id.
   *
   * @param string $service_id
   */
  public function clearBinCache($service_id) {
    \Drupal::service($service_id)->deleteAll();
  }

  /**
   * List of the core cache bin.
   */
  public function coreBinMapping() {
    $core_bins = array(
      'bootstrap',
      'config',
      'data',
      'default',
      'discovery',
      'dynamic_page_cache',
      'entity',
      'menu',
      'render',
      'migrate',
      'rest',
      'toolbar',
    );
    return $core_bins;
  }

  /**
   * Check if entity exists and is enabled. 
   * 
   * @param CacheflushEntity $entity
   *
   * @return bool
   */
  private function checkError($entity) {
    if (!$entity) {
      drupal_set_message(t('Invalid entity ID.'), 'error');
      throw new HttpException('404');
    }
    if ($entity->getStatus() == 0) {
      drupal_set_message(t('This entity is disabled.'), 'error');
      throw new HttpException('403');
    }
  }

  /**
   * Generate redirect URL. 
   *
   * @global string $base_url
   *
   * @return RedirectResponse
   */
  private function redirectUrl() {
    global $base_url;

    $path = isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : NULL;
    if (empty($_SERVER['HTTP_REFERER'])) {
      $path = $base_url;
    }
    return new RedirectResponse($path);
  }

}
